package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.OrgaoComprovanteCustomizadoDAO;
import com.rjconsultores.ventaboletos.entidad.OrgaoComprovanteCustomizado;
import com.rjconsultores.ventaboletos.entidad.OrgaoConcedente;

@Repository("orgaoComprovanteCustomizadoDAO")
public class OrgaoComprovanteCustomizadoHibernateDAO extends GenericHibernateDAO<OrgaoComprovanteCustomizado, Integer>
		implements OrgaoComprovanteCustomizadoDAO {

	@Autowired
	public OrgaoComprovanteCustomizadoHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<OrgaoComprovanteCustomizado> obtenerTodos() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

		return c.list();
	}
	
	public List<OrgaoComprovanteCustomizado> buscarOrgaoPorOgao(OrgaoConcedente orgaoConcedente) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("orgaoConcedente", orgaoConcedente));
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

		return c.list();
	}

	@Override
	public OrgaoComprovanteCustomizado buscarPorOrgao(OrgaoConcedente orgaoConcedente, String cveTipoComprovante) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		c.add(Restrictions.eq("orgaoConcedente", orgaoConcedente));
		c.add(Restrictions.eq("cveTipoComprovante", cveTipoComprovante));

		return (OrgaoComprovanteCustomizado) c.uniqueResult();
	}
	
	@Override
	public OrgaoComprovanteCustomizado buscarPorOrgaoPorTipoComprovanteId(OrgaoConcedente orgaoConcedente, Integer tipoComprovanteId) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("orgaoConcedente", orgaoConcedente));
		c.add(Restrictions.eq("tipoComprovanteId", tipoComprovanteId));

		return (OrgaoComprovanteCustomizado) c.uniqueResult();
	}
}
