package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.PacoteTarifaDAO;
import com.rjconsultores.ventaboletos.entidad.Pacote;
import com.rjconsultores.ventaboletos.entidad.PacoteTarifa;
import com.rjconsultores.ventaboletos.entidad.TipoTarifaPacote;

@Repository("pacoteTarifaDAO")
public class PacoteTarifaHibernateDAO extends GenericHibernateDAO<PacoteTarifa, Integer> implements PacoteTarifaDAO {

	@Autowired
	public PacoteTarifaHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public PacoteTarifa buscaPacoteTarifa(Pacote pacote, TipoTarifaPacote tipoTarifaPacote) {

		StringBuilder sb = new StringBuilder();
		sb.append(" from PacoteTarifa pt ");
		sb.append(" where pt.pacote.pacoteId = :pacoteId ");
		sb.append("   and pt.tipoTarifaPacote.tipotarifapacoteId = :tipotarifapacoteId ");
		sb.append("   and pt.activo = 1 ");

		Query query = getSession().createQuery(sb.toString());
		query.setInteger("pacoteId", pacote.getPacoteId());
		query.setInteger("tipotarifapacoteId", tipoTarifaPacote.getTipotarifapacoteId());

		return (PacoteTarifa) query.uniqueResult();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<PacoteTarifa> buscaTarifasPacote(Pacote pacote) {

		StringBuilder sb = new StringBuilder();
		sb.append(" from PacoteTarifa pt ");
		sb.append(" where pt.pacote.pacoteId = :pacoteId ");
		sb.append("   and pt.activo = 1 ");

		Query query = getSession().createQuery(sb.toString());
		query.setInteger("pacoteId", pacote.getPacoteId());

		return query.list();
	}

}
