package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * 
 * @author wallace
 */
@Entity
@SequenceGenerator(name = "CONEXION_RUTA_CTRL_SEQ", sequenceName = "CONEXION_RUTA_CTRL_SEQ", allocationSize = 1)
@Table(name = "CONEXION_RUTA_CTRL")
public class ConexionRutaCtrl implements Serializable {
	private static final long serialVersionUID = 1L;
	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "CONEXION_RUTA_CTRL_SEQ")
	@Column(name = "CONEXIONRUTACTRL_ID")
	private Long conexionRutaId;
	@Column(name = "ACTIVO")
	private Boolean activo;
	@Column(name = "FECMODIF")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecmodif;
	@Column(name = "USUARIO_ID")
	private Integer usuarioId;
	
	@ManyToOne
	@JoinColumn(name = "RUTA_ORIGEN_ID")
	private Ruta rutaOrigen;
	
	@ManyToOne
	@JoinColumn(name = "RUTA_DESTINO_ID")
	private Ruta rutaDestino;
	
	@ManyToOne
	@JoinColumn(name = "RUTA_DESTINOC_ID")
	private Ruta rutaDestinoC;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "CONEXIONRUTACONF_ID")
	private ConexionRutaConf conexionRutaConf;

	public ConexionRutaCtrl() {
		super();
	}

	public Boolean getActivo() {
		return activo;
	}

	public Long getConexionRutaId() {
		return conexionRutaId;
	}

	public void setConexionRutaId(Long conexionRutaId) {
		this.conexionRutaId = conexionRutaId;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Integer getUsuarioId() {
		return usuarioId;
	}

	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}

	public ConexionRutaConf getConexionRutaConf() {
		return conexionRutaConf;
	}

	public void setConexionRutaConf(ConexionRutaConf conexionRutaConf) {
		this.conexionRutaConf = conexionRutaConf;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((conexionRutaId == null) ? 0 : conexionRutaId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ConexionRutaCtrl other = (ConexionRutaCtrl) obj;
		if (conexionRutaId == null) {
			if (other.conexionRutaId != null)
				return false;
		} else if (!conexionRutaId.equals(other.conexionRutaId))
			return false;
		return true;
	}
	
	public static ConexionRutaCtrl getConexionRutaCtrl(List<ConexionRutaCtrl> list, Integer... rutas) {
		if(rutas != null && rutas.length > 1) {
			for (ConexionRutaCtrl conexionRutaCtrl : list) {
				boolean isMatched = conexionRutaCtrl.getRutaOrigen() != null && conexionRutaCtrl.getRutaOrigen().getRutaId().equals(rutas[0]);
				if(!isMatched) {
					continue;
				}
				
				isMatched = conexionRutaCtrl.getRutaDestino() != null && conexionRutaCtrl.getRutaDestino().getRutaId().equals(rutas[1]);
				if(!isMatched) {
					continue;
				}
				
				if(rutas.length > 2 && conexionRutaCtrl.getRutaDestinoC() != null) {
					isMatched = conexionRutaCtrl.getRutaDestinoC().getRutaId().equals(rutas[2]);
				}
				
				if(isMatched) {
					return conexionRutaCtrl;
				}
			}
		}
		throw new RuntimeException("ConexionRutaCtrl não identificada");
	}

	public Ruta getRutaOrigen() {
		return rutaOrigen;
	}

	public void setRutaOrigen(Ruta rutaOrigen) {
		this.rutaOrigen = rutaOrigen;
	}

	public Ruta getRutaDestino() {
		return rutaDestino;
	}

	public void setRutaDestino(Ruta rutaDestino) {
		this.rutaDestino = rutaDestino;
	}

	public Ruta getRutaDestinoC() {
		return rutaDestinoC;
	}

	public void setRutaDestinoC(Ruta rutaDestinoC) {
		this.rutaDestinoC = rutaDestinoC;
	}
	
}
