package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.Hibernate;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.AuditarEntidade;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@Entity
@AuditarClasse(nome = "ConfRestricaoTramo", tela = "auditarClasse.ConfRestricaoTramo")
@SequenceGenerator(name = "CONF_RESTRICAO_TRAMO_SEQ", sequenceName = "CONF_RESTRICAO_TRAMO_SEQ", allocationSize = 1)
@Table(name = "CONF_RESTRICAO_TRAMO")
public class ConfRestricaoTramo implements Serializable,  Auditavel<ConfRestricaoTramo> {

	private static final long serialVersionUID = 1L;

	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "CONF_RESTRICAO_TRAMO_SEQ")
	@Column(name = "CONFRESTRICAOTRAMO_ID")
	private Long confRestricaoTramoId;

	@JoinColumn(name = "CONFRESTRICAOCANALVENTA_ID", referencedColumnName = "CONFRESTRICAOCANALVENTA_ID")
	@ManyToOne
	private ConfRestricaoCanalVenta confRestricaoCanalVenta;
	@AuditarEntidade
	@OneToOne
	@JoinColumn(name = "TIPOPTOVTA_ID")
	private TipoPuntoVenta tipoPuntoVenta;
	@OneToOne
	@JoinColumn(name = "ORIGEN_ID")
	private Parada origem;
	@OneToOne
	@JoinColumn(name = "DESTINO_ID")
	private Parada destino;
	@Column(name = "TEMPOPERMITIR")
	private Date tempoPermitir;

	@Column(name = "SEGUNDA")
	private Boolean segunda;
	@Column(name = "TERCA")
	private Boolean terca;
	@Column(name = "QUARTA")
	private Boolean quarta;
	@Column(name = "QUINTA")
	private Boolean quinta;
	@Column(name = "SEXTA")
	private Boolean sexta;
	@Column(name = "SABADO")
	private Boolean sabado;
	@Column(name = "DOMINGO")
	private Boolean domingo;
	@Column(name = "ACTIVO")
	private Boolean activo;
	@Column(name = "FECMODIF")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecmodif;
	@Column(name = "USUARIO_ID")
	private Integer usuarioId;
	@Transient
	@NaoAuditar
	private ConfRestricaoTramo confRestricaoTramoClone; 

	public Long getConfRestricaoTramoId() {
		return confRestricaoTramoId;
	}

	public void setConfRestricaoTramoId(Long confRestricaoTramoId) {
		this.confRestricaoTramoId = confRestricaoTramoId;
	}

	public ConfRestricaoCanalVenta getConfRestricaoCanalVenta() {
		return confRestricaoCanalVenta;
	}

	public void setConfRestricaoCanalVenta(ConfRestricaoCanalVenta confRestricaoCanalVenta) {
		this.confRestricaoCanalVenta = confRestricaoCanalVenta;
	}

	public TipoPuntoVenta getTipoPuntoVenta() {
		return tipoPuntoVenta;
	}

	public void setTipoPuntoVenta(TipoPuntoVenta tipoPuntoVenta) {
		this.tipoPuntoVenta = tipoPuntoVenta;
	}

	public Parada getOrigem() {
		return origem;
	}

	public void setOrigem(Parada origem) {
		this.origem = origem;
	}

	public Parada getDestino() {
		return destino;
	}

	public void setDestino(Parada destino) {
		this.destino = destino;
	}

	public Date getTempoPermitir() {
		return tempoPermitir;
	}

	public void setTempoPermitir(Date tempoPermitir) {
		this.tempoPermitir = tempoPermitir;
	}

	public Boolean getSegunda() {
		return segunda;
	}

	public void setSegunda(Boolean segunda) {
		this.segunda = segunda;
	}

	public Boolean getTerca() {
		return terca;
	}

	public void setTerca(Boolean terca) {
		this.terca = terca;
	}

	public Boolean getQuarta() {
		return quarta;
	}

	public void setQuarta(Boolean quarta) {
		this.quarta = quarta;
	}

	public Boolean getQuinta() {
		return quinta;
	}

	public void setQuinta(Boolean quinta) {
		this.quinta = quinta;
	}

	public Boolean getSexta() {
		return sexta;
	}

	public void setSexta(Boolean sexta) {
		this.sexta = sexta;
	}

	public Boolean getSabado() {
		return sabado;
	}

	public void setSabado(Boolean sabado) {
		this.sabado = sabado;
	}

	public Boolean getDomingo() {
		return domingo;
	}

	public void setDomingo(Boolean domingo) {
		this.domingo = domingo;
	}

	public Boolean getActivo() {
		return activo;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Integer getUsuarioId() {
		return usuarioId;
	}

	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		if (confRestricaoTramoId != null)
			builder.append(confRestricaoTramoId).append(" - ");
		if (tipoPuntoVenta != null)
			builder.append(tipoPuntoVenta).append(" - ");
		if (origem != null)
			builder.append(origem).append(" - ");
		if (destino != null)
			builder.append(destino);
		return builder.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((activo == null) ? 0 : activo.hashCode());
		result = prime * result + ((confRestricaoCanalVenta == null) ? 0 : confRestricaoCanalVenta.hashCode());
		result = prime * result + ((destino == null) ? 0 : destino.hashCode());
		result = prime * result + ((origem == null) ? 0 : origem.hashCode());
		result = prime * result + ((tipoPuntoVenta == null) ? 0 : tipoPuntoVenta.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof ConfRestricaoTramo)) {
			return false;
		}
		ConfRestricaoTramo other = (ConfRestricaoTramo) obj;
		if (activo == null) {
			if (other.activo != null) {
				return false;
			}
		} else if (!activo.equals(other.activo)) {
			return false;
		}
		if (confRestricaoCanalVenta == null) {
			if (other.confRestricaoCanalVenta != null) {
				return false;
			}
		} else if (!confRestricaoCanalVenta.equals(other.confRestricaoCanalVenta)) {
			return false;
		}
		if (destino == null) {
			if (other.destino != null) {
				return false;
			}
		} else if (!destino.equals(other.destino)) {
			return false;
		}
		if (origem == null) {
			if (other.origem != null) {
				return false;
			}
		} else if (!origem.equals(other.origem)) {
			return false;
		}
		if (tipoPuntoVenta == null) {
			if (other.tipoPuntoVenta != null) {
				return false;
			}
		} else if (!tipoPuntoVenta.equals(other.tipoPuntoVenta)) {
			return false;
		}
		return true;
	}
	
	@Override
	public void clonar() throws CloneNotSupportedException {
		
		confRestricaoTramoClone = new ConfRestricaoTramo();
		confRestricaoTramoClone = (ConfRestricaoTramo) this.clone();
		Hibernate.initialize(confRestricaoTramoClone.getConfRestricaoCanalVenta());
		
	}

	@Override
	public ConfRestricaoTramo getCloneObject() throws CloneNotSupportedException {
		return confRestricaoTramoClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getOrigem().getDescparada()+"-"+getDestino().getDescparada());
	}

}
