package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.AuditarID;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@Entity
@Table(name = "OPERADOR_EMBARCADA_LINHA")
@AuditarClasse(nome = "OPERADOR_EMBARCADA_LINHA", tela = "auditarClasse.OperadorEmbarcadaLinha")
@SequenceGenerator(name = "OPERADOR_EMBARCADA_LINHA_SEQ", sequenceName = "OPERADOR_EMBARCADA_LINHA_SEQ", allocationSize = 1)
public class OperadorEmbarcadaLinha implements Serializable, Auditavel<OperadorEmbarcadaLinha>  {

	private static final long serialVersionUID = 1L;
	
	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "OPERADOR_EMBARCADA_LINHA_SEQ")
	@Column(name = "OPERADOR_EMBARCADA_LINHA_ID")
	@AuditarID
	private Long operadorEmbarcadaLinhaId;
	
	@ManyToOne
	@JoinColumn(name = "OPERADOR_EMBARCADA_ID", referencedColumnName = "OPERADOR_EMBARCADA_ID")
	private OperadorEmbarcada operadorEmbarcada;
	
	@OneToOne
	@JoinColumn(name = "RUTA_ID")
	private Ruta ruta;
	
	@Column(name = "ACTIVO")
	private Boolean activo;
	
	@Transient
	@NaoAuditar
	private OperadorEmbarcadaLinha operadorClone;

	public Long getOperadorEmbarcadaLinhaId() {
		return operadorEmbarcadaLinhaId;
	}

	public void setOperadorEmbarcadaLinhaId(Long operadorEmbarcadaLinhaId) {
		this.operadorEmbarcadaLinhaId = operadorEmbarcadaLinhaId;
	}

	public OperadorEmbarcada getOperadorEmbarcada() {
		return operadorEmbarcada;
	}

	public void setOperadorEmbarcada(OperadorEmbarcada operadorEmbarcada) {
		this.operadorEmbarcada = operadorEmbarcada;
	}

	public Boolean getActivo() {
		return activo == null ? false : activo;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Ruta getRuta() {
		return ruta;
	}

	public void setRuta(Ruta ruta) {
		this.ruta = ruta;
	}
	
	@Override
	public void clonar() throws CloneNotSupportedException {
		operadorClone = new OperadorEmbarcadaLinha();
		operadorClone = (OperadorEmbarcadaLinha) this.clone();
	}

	@Override
	public OperadorEmbarcadaLinha getCloneObject() throws CloneNotSupportedException {
		return operadorClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getOperadorEmbarcadaLinhaId() + "- linha: " + ruta.getRutaId() + " - " + ruta.getDescruta());
	}

	@Override
	public String toString() {
		return ruta.getNumRuta();
	}
	
	
	
}
