package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CasetaPeajeExcepcionDAO;
import com.rjconsultores.ventaboletos.entidad.CasetaPeaje;
import com.rjconsultores.ventaboletos.entidad.CasetaPeajeExcepcion;
import com.rjconsultores.ventaboletos.service.CasetaPeajeExcepcionService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("casetaPeajeExcepcionService")
public class CasetaPeajeExcepcionServiceImpl implements CasetaPeajeExcepcionService {

	@Autowired
	private CasetaPeajeExcepcionDAO casetaPeajeExcepcionDAO;
	
	@Override
	public List<CasetaPeajeExcepcion> buscarPorCasetaPeaje(CasetaPeaje caseta){
		return casetaPeajeExcepcionDAO.obtenerPorCasetaPeaje(caseta);
	}

	@Override
	public List<CasetaPeajeExcepcion> obtenerTodos() {
		return casetaPeajeExcepcionDAO.obtenerTodos();
	}

	@Override
	public CasetaPeajeExcepcion obtenerID(Integer id) {
		return casetaPeajeExcepcionDAO.obtenerID(id);
	}

	@Transactional
	public CasetaPeajeExcepcion suscribir(CasetaPeajeExcepcion entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		return casetaPeajeExcepcionDAO.suscribir(entidad);
	}

	@Transactional
	public CasetaPeajeExcepcion actualizacion(CasetaPeajeExcepcion entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		return casetaPeajeExcepcionDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(CasetaPeajeExcepcion entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);
		casetaPeajeExcepcionDAO.borrar(entidad);
	}
}
