package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CoeficienteTarifaDAO;
import com.rjconsultores.ventaboletos.dao.OrgaoTramoDAO;
import com.rjconsultores.ventaboletos.entidad.CoeficienteTarifa;
import com.rjconsultores.ventaboletos.service.CoeficienteTarifaService;
import com.rjconsultores.ventaboletos.service.LogAuditoriaService;
import com.rjconsultores.ventaboletos.utilerias.RegistroConDependenciaException;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("coeficienteTarifaService")
public class CoeficienteTarifaServiceImpl implements CoeficienteTarifaService {

	@Autowired
	private CoeficienteTarifaDAO coeficienteTarifaDAO;
	@Autowired
	private OrgaoTramoDAO orgaoTramoDAO;
	@Autowired
	private LogAuditoriaService logAuditoriaService;
	private static final Logger log = LogManager.getLogger(OrgaoConcedenteServiceImpl.class);


	public List<CoeficienteTarifa> obtenerTodos() {
		return coeficienteTarifaDAO.obtenerTodos();
	}

	public CoeficienteTarifa obtenerID(Integer id) {
		
		CoeficienteTarifa coeficienteTarifa = coeficienteTarifaDAO.obtenerID(id);

		try {
			coeficienteTarifa.clonar();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return coeficienteTarifa;
		
	}

	@Transactional
	public CoeficienteTarifa suscribir(CoeficienteTarifa entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		
		entidad =  coeficienteTarifaDAO.suscribir(entidad);
		logAuditoriaService.auditar(null, entidad, null);
		return entidad;
	}

	@Transactional
	public CoeficienteTarifa actualizacion(CoeficienteTarifa entidad) {
		
						
		try {
			
			entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
			entidad.setFecmodif(Calendar.getInstance().getTime());
			entidad.setActivo(Boolean.TRUE);
			
			CoeficienteTarifa originalClone = entidad.getCloneObject();
			entidad = coeficienteTarifaDAO.actualizacion(entidad);
			
			logAuditoriaService.auditar(originalClone, entidad, null);

			
		} catch (Exception e) {
			log.error(e);
		}
		
		return entidad;
	}

	@Transactional
	public void borrar(CoeficienteTarifa entidad) throws RegistroConDependenciaException {
		long count =0; 
		count +=orgaoTramoDAO.count("coeficienteTarifa1", entidad); 
		count +=orgaoTramoDAO.count("coeficienteTarifa2", entidad); 
		count +=orgaoTramoDAO.count("coeficienteTarifa3", entidad); 
		
		if (count >0 ){
			throw new RegistroConDependenciaException();
		}
		
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);
				
		entidad = coeficienteTarifaDAO.actualizacion(entidad);
		logAuditoriaService.auditarExclusao(entidad, null);
	}

	public List<CoeficienteTarifa> buscar(String nomb) {
		return coeficienteTarifaDAO.buscar(nomb);
	}
}
