package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CursoDAO;
import com.rjconsultores.ventaboletos.entidad.Curso;
import com.rjconsultores.ventaboletos.service.CursoService;
import com.rjconsultores.ventaboletos.utilerias.RegistroConDependenciaException;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("cursoService")
public class CursoServiceImpl implements CursoService {

	@Autowired
	private CursoDAO CursoDAO;

	public List<Curso> obtenerTodos() {
		return CursoDAO.obtenerTodos();
	}

	public Curso obtenerID(Integer id) {
		return CursoDAO.obtenerID(id);
	}

	@Transactional
	public Curso suscribir(Curso entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return CursoDAO.suscribir(entidad);
	}

	@Transactional
	public Curso actualizacion(Curso entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return CursoDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(Curso entidad) throws RegistroConDependenciaException {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		CursoDAO.actualizacion(entidad);
	}
	
	
	public List<Curso> buscar(String nomcurso){
		return CursoDAO.buscar(nomcurso);
	}

}
