
package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.EmpresaSicfeConfigDAO;
import com.rjconsultores.ventaboletos.entidad.EmpresaSicfeConfig;
import com.rjconsultores.ventaboletos.service.EmpresaSicfeConfigService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("empresaSicfeConfigService")
public class EmpresaSicfeConfigServiceImpl implements EmpresaSicfeConfigService {

	@Autowired
	private EmpresaSicfeConfigDAO empresaSicfeConfigDAO;

	public List<EmpresaSicfeConfig> obtenerTodos() {
		return empresaSicfeConfigDAO.obtenerTodos();
	}

	public EmpresaSicfeConfig obtenerID(Integer id) {
		return empresaSicfeConfigDAO.obtenerID(id);
	}

	@Transactional
	public EmpresaSicfeConfig suscribir(EmpresaSicfeConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaSicfeConfigDAO.suscribir(entidad);
	}

	@Transactional
	public EmpresaSicfeConfig actualizacion(EmpresaSicfeConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaSicfeConfigDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(EmpresaSicfeConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		empresaSicfeConfigDAO.actualizacion(entidad);
	}

	public EmpresaSicfeConfig buscarChave(String chave, Integer empresaId) {
		return empresaSicfeConfigDAO.buscarChave(chave, empresaId);
	}

	public List<EmpresaSicfeConfig> buscarByEmpresa(Integer empresaId) {
		return empresaSicfeConfigDAO.buscarByEmpresa(empresaId);
	}

}
