package com.rjconsultores.ventaboletos.service.impl;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.PacoteTarifaDAO;
import com.rjconsultores.ventaboletos.entidad.Categoria;
import com.rjconsultores.ventaboletos.entidad.Pacote;
import com.rjconsultores.ventaboletos.entidad.PacoteTarifa;
import com.rjconsultores.ventaboletos.entidad.TipoTarifaPacote;
import com.rjconsultores.ventaboletos.service.PacoteTarifaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("pacoteTarifaService")
public class PacoteTarifaServiceImpl implements PacoteTarifaService {

	@Autowired
	private PacoteTarifaDAO pacoteTarifaDAO;

	@Override
	public List<PacoteTarifa> obtenerTodos() {
		return pacoteTarifaDAO.obtenerTodos();
	}

	@Override
	public PacoteTarifa obtenerID(Integer id) {
		return pacoteTarifaDAO.obtenerID(id);
	}

	@Transactional
	@Override
	public PacoteTarifa suscribir(PacoteTarifa entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return pacoteTarifaDAO.suscribir(entidad);
	}

	@Transactional
	@Override
	public PacoteTarifa actualizacion(PacoteTarifa entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return pacoteTarifaDAO.actualizacion(entidad);
	}

	@Transactional
	@Override
	public void borrar(PacoteTarifa entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		pacoteTarifaDAO.actualizacion(entidad);
	}

	@Transactional(readOnly = false)
	@Override
	public PacoteTarifa vincularPacoteTarifa(PacoteTarifa pacoteTarifa, Pacote pacote, TipoTarifaPacote tipoTarifaPacote, BigDecimal tarifa, Boolean indvendaweb, Categoria categoria) {

		if (pacoteTarifa == null) {
			pacoteTarifa = new PacoteTarifa();
		}

		pacoteTarifa.setIndvendaweb(indvendaweb);
		pacoteTarifa.setTarifa(tarifa);
		pacoteTarifa.setTipoTarifaPacote(tipoTarifaPacote);
		pacoteTarifa.setPacote(pacote);
		pacoteTarifa.setCategoria(categoria);

		if (pacoteTarifa.getPacotetarifaId() == null) {
			return suscribir(pacoteTarifa);
		} else {
			return actualizacion(pacoteTarifa);
		}
	}

	@Override
	public PacoteTarifa buscaPacoteTarifa(Pacote pacote, TipoTarifaPacote tipoTarifaPacote) {
		return pacoteTarifaDAO.buscaPacoteTarifa(pacote, tipoTarifaPacote);
	}

	@Override
	public List<PacoteTarifa> buscaTarifasPacote(Pacote pacote) {
		return pacoteTarifaDAO.buscaTarifasPacote(pacote);
	}
}
